"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _stanUtils = require("stan-utils");

var _writeDts = _interopRequireDefault(require("./write-dts"));

var _writeReference = _interopRequireDefault(require("./write-reference"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * proto generate dts
 * @param opts
 * @returns parsedFiles path
 */
function protoGenDTS(opts) {
  if (!opts.files.length) {
    throw new Error('Please enter the proto file');
  }

  var parsedFiles = [];

  var _iterator = _createForOfIteratorHelper(opts.files),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var file = _step.value;
      var readablyFile = (0, _stanUtils.relativeNormalize)(file.file);

      var _file = _objectSpread({}, file);

      var _path$parse = _path.default.parse(file.file),
          dir = _path$parse.dir,
          name = _path$parse.name;

      if (!_file.output) {
        _file.output = _path.default.join(dir, name + '.d.ts');
      }

      if ((0, _stanUtils.slash)(_file.output).slice(-1) === '/') {
        _file.output = _path.default.join(_file.output, name + '.d.ts');
      }

      var startTime = Date.now();
      console.log("Compile ".concat(_stanUtils.chalk.yellow(readablyFile), " \u2192 ").concat(_stanUtils.chalk.cyan((0, _stanUtils.relativeNormalize)(_file.output)), "..."));

      try {
        var dts = (0, _writeDts.default)(_file, _objectSpread(_objectSpread({
          alternateCommentMode: true
        }, opts.protoParseOptions), file.protoParseOptions), opts.visitor);
        parsedFiles.push.apply(parsedFiles, _toConsumableArray(dts));

        if (dts.length) {
          var spendMS = Date.now() - startTime;
          console.log("Created ".concat(_stanUtils.chalk.cyan((0, _stanUtils.relativeNormalize)(_file.output)), " in ").concat(_stanUtils.chalk.bold((0, _stanUtils.pms)(spendMS))));
        }

        if (dts.length > 1) {
          console.log("  Created dependent modules: \n" + _stanUtils.chalk.greenBright(dts.slice(1).map(_stanUtils.relativeNormalize).map(function (v) {
            return '  • ' + v;
          }).join('\n')));
        }
      } catch (e) {
        console.log("Build ".concat(_stanUtils.chalk.red(readablyFile), " error: "), e);
        throw e;
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  if (opts.referenceEntryFile !== false) {
    var referenceEntryFile = opts.referenceEntryFile || 'index.d.ts';
    (0, _writeReference.default)(parsedFiles, referenceEntryFile);
    console.log("Created reference entry file: ".concat(_stanUtils.chalk.cyan((0, _stanUtils.relativeNormalize)(referenceEntryFile))));
  }

  return parsedFiles;
}

var _default = protoGenDTS;
exports.default = _default;