"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = writeDTS;

var _path = _interopRequireDefault(require("path"));

var _googleProtoFiles = require("google-proto-files");

var _protobufjs = require("protobufjs");

var _stanUtils = require("stan-utils");

var _util = require("./util");

var _parseNamespace = _interopRequireDefault(require("./parse-namespace"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * record parsed files
 */
var parsedFiles = [];
/**
 * generate dts file
 * @param proto
 * @param opts
 * @param visitor
 * @returns generate files
 */

function writeDTS(proto, opts, visitor) {
  var _proto$generateDepend = proto.generateDependentModules,
      generateDependentModules = _proto$generateDepend === void 0 ? true : _proto$generateDepend;

  if (parsedFiles.includes(proto.file)) {
    return [];
  }

  function loadFile(file) {
    var root = (0, _googleProtoFiles.loadSync)(file, opts);

    if (!root.nestedArray.length) {
      console.log(_stanUtils.chalk.yellowBright("Warning \"".concat(proto.file, "\" is empty")));
    }

    return root;
  } // only compile namespace
  // false where throw error


  function checkCanCompile(reflection) {
    var can = reflection instanceof _protobufjs.Namespace;

    if (!can) {
      console.log(_stanUtils.chalk.yellowBright("Check whether the version of protobufjs that google-proto-files depends on is consistent with the version of protobufjs in the project."));
      throw new Error((0, _util.reportIssues)({
        title: "Type ".concat(reflection === null || reflection === void 0 ? void 0 : reflection.name, " not supported.")
      }));
    }

    return can;
  }

  var root = loadFile(proto.file);
  /**
   * generated dts files path
   */

  var generatedFiles = [];
  var reflection = root.nestedArray[0];

  if (!reflection) {
    return generatedFiles;
  }

  if (checkCanCompile(reflection)) {
    var outPath = _path.default.resolve(proto.output);

    var parsed = (0, _parseNamespace.default)(reflection, root.files[0], visitor);

    _stanUtils.fs.outputFileSync(outPath, (0, _util.writeBanner)(parsed));

    generatedFiles.push(outPath);
    parsedFiles.push(proto.file);
    var protoLibDir = (0, _googleProtoFiles.getProtoPath)();

    if (generateDependentModules && root.files.length > 1) {
      var _iterator = _createForOfIteratorHelper(root.files.slice(1)),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var file = _step.value;

          if (parsedFiles.includes(file)) {
            continue;
          }

          var orgFile = file;

          var _root = loadFile(file);

          var _reflection = _root.nestedArray[0];

          if (!_reflection) {
            continue;
          }

          var isLibFile = file.startsWith(protoLibDir);

          if (checkCanCompile(_reflection)) {
            var _parsed = (0, _parseNamespace.default)(_reflection, file, visitor);

            var _path$parse = _path.default.parse(proto.output),
                dir = _path$parse.dir; // internal proto file, beautify the generation path


            if (file.startsWith(protoLibDir)) {
              file = (0, _util.replaceSamePath)(protoLibDir, file);
            } // beautify the proto lib path


            if (isLibFile) {
              file = _path.default.join('google', file);
            } // generate a file based on the relative position of the file


            var _path$parse2 = _path.default.parse((0, _util.replaceSamePath)(proto.file, file)),
                importDir = _path$parse2.dir,
                fileName = _path$parse2.name;

            var _outPath = _path.default.resolve(dir, importDir, "".concat(fileName, ".d.ts"));

            _stanUtils.fs.outputFileSync(_outPath, (0, _util.writeBanner)(_parsed));

            generatedFiles.push(_outPath);
            parsedFiles.push(orgFile);
          }
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
    }
  }

  return generatedFiles;
}